@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.207 $
+* $Revision: 1.232 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -58,9 +58,14 @@
 #define ETIMEDOUT WSAETIMEDOUT
 #define EADDRNOTAVAIL WSAEADDRNOTAVAIL
 #define ECONNABORTED WSAECONNABORTED
+#define ENETUNREACH WSAENETUNREACH
 
 #endif /* _WIN32 */
 
+#ifdef HAVE_C_ARES
+#include <ares.h>
+#endif /* HAVE_C_ARES */
+
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
@@ -79,6 +84,7 @@
 #endif /* __sun */
 
 #include <vector>
+#include <list>
 #include <iostream>
 #include <sstream>
 #include <string>
@@ -111,6 +117,21 @@
 #endif /* __DEBUG__ */
 #endif /* PERROR */
 
+#ifdef _WIN32
+typedef SOCKET cs_sock_t;
+#ifdef _WIN64
+typedef signed __int64 cs_ssize_t;
+#else
+typedef signed int cs_ssize_t;
+#endif  /* _WIN64 */
+#define CS_INVALID_SOCK	INVALID_SOCKET
+#else
+typedef int cs_sock_t;
+typedef ssize_t cs_ssize_t;
+#define CS_INVALID_SOCK	-1
+#endif /* _WIN32 */
+
+
 #ifndef _NO_CSOCKET_NS // some people may not want to use a namespace
 namespace Csocket
 {
@@ -300,12 +321,12 @@ inline void TFD_ZERO( fd_set *set )
 	FD_ZERO( set );
 }
 
-inline void TFD_SET( u_int iSock, fd_set *set )
+inline void TFD_SET( cs_sock_t iSock, fd_set *set )
 {
 	FD_SET( iSock, set );
 }
 
-inline bool TFD_ISSET( u_int iSock, fd_set *set )
+inline bool TFD_ISSET( cs_sock_t iSock, fd_set *set )
 {
 	if ( FD_ISSET( iSock, set ) )
 		return( true );
@@ -313,7 +334,7 @@ inline bool TFD_ISSET( u_int iSock, fd_set *set )
 	return( false );
 }
 
-inline void TFD_CLR( u_int iSock, fd_set *set )
+inline void TFD_CLR( cs_sock_t iSock, fd_set *set )
 {
 	FD_CLR( iSock, set );
 }
@@ -408,10 +429,10 @@ public:
 	* Advanced constructor, for creating a simple connection
 	*
 	* @param sHostname the hostname your are connecting to
-	* @param iport the port you are connectint to
+	* @param uPort the port you are connectint to
 	* @param itimeout how long to wait before ditching the connection, default is 60 seconds
 	*/
-	Csock( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
+	Csock( const CS_STRING & sHostname, u_short uPort, int itimeout = 60 );
 
 	//! override this for accept sockets
 	virtual Csock *GetSockObj( const CS_STRING & sHostname, u_short iPort );
@@ -521,7 +542,7 @@ public:
 	virtual bool Listen( u_short iPort, int iMaxConns = SOMAXCONN, const CS_STRING & sBindHost = "", u_int iTimeout = 0 );
 
 	//! Accept an inbound connection, this is used internally
-	virtual int Accept( CS_STRING & sHost, u_short & iRPort );
+	virtual cs_sock_t Accept( CS_STRING & sHost, u_short & iRPort );
 
 	//! Accept an inbound SSL connection, this is used internally and called after Accept
 	virtual bool AcceptSSL();
@@ -551,7 +572,7 @@ public:
 	* @param len the length of data
 	*
 	*/
-	virtual bool Write( const char *data, int len );
+	virtual bool Write( const char *data, size_t len );
 
 	/**
 	* convience function
@@ -574,26 +595,28 @@ public:
 	* Returns READ_TIMEDOUT for a connection that timed out at the TCP level
 	* Otherwise returns the bytes read into data
 	*/
-	virtual int Read( char *data, int len );
+	virtual cs_ssize_t Read( char *data, size_t len );
 	CS_STRING GetLocalIP();
 	CS_STRING GetRemoteIP();
 
+	virtual CS_STRING ConvertAddress( void *addr, bool bIPv6 = false );
+
 	//! Tells you if the socket is connected
 	virtual bool IsConnected();
 	//! Sets the sock, telling it its connected (internal use only)
 	virtual void SetIsConnected( bool b );
 
 	//! returns a reference to the sock
-	int & GetRSock();
-	void SetRSock( int iSock );
-	int & GetWSock();
-	void SetWSock( int iSock );
+	cs_sock_t & GetRSock();
+	void SetRSock( cs_sock_t iSock );
+	cs_sock_t & GetWSock();
+	void SetWSock( cs_sock_t iSock );
 
-	void SetSock( int iSock );
-	int & GetSock();
+	void SetSock( cs_sock_t iSock );
+	cs_sock_t & GetSock();
 
-	//! resets the time counter
-	void ResetTimer();
+	//! resets the time counter, this is virtual in the event you need an event on the timer being Reset
+	virtual void ResetTimer();
 
 	//! will pause/unpause reading on this socket
 	void PauseRead();
@@ -627,7 +650,7 @@ public:
 	* pushes data up on the buffer, if a line is ready
 	* it calls the ReadLine event
 	*/
-	virtual void PushBuff( const char *data, int len, bool bStartAtZero = false );
+	virtual void PushBuff( const char *data, size_t len, bool bStartAtZero = false );
 
 	//! This gives access to the internal read buffer, if your
 	//! not going to use ReadLine(), then you may want to clear this out
@@ -744,8 +767,11 @@ public:
 #ifdef HAVE_LIBSSL
 	X509 *getX509();
 
-	//! Returns The Peers Public Key
+	//! Returns the peer's public key
 	CS_STRING GetPeerPubKey();
+	//! Returns the peer's certificate finger print
+	int GetPeerFingerprint( CS_STRING & sFP);
+
 	unsigned int GetRequireClientCertFlags();
 	//! legacy, deprecated @see SetRequireClientCertFlags
 	void SetRequiresClientCert( bool bRequiresCert );
@@ -815,7 +841,7 @@ public:
 	*
 	* Ready to read data event
 	*/
-	virtual void ReadData( const char *data, int len ) {}
+	virtual void ReadData( const char *data, size_t len ) {}
 	/**
 	* Override these functions for an easy interface when using the Socket Manager
 	* Don't bother using these callbacks if you are using this class directly (without Socket Manager)
@@ -884,7 +910,7 @@ public:
 
 
 	//! return how long it has been (in seconds) since the last read or successful write
-	int GetTimeSinceLastDataTransaction( time_t iNow = 0 )
+	time_t GetTimeSinceLastDataTransaction( time_t iNow = 0 )
 	{
 		if( m_iLastCheckTimeoutTime == 0 )
 			return( 0 );
@@ -905,30 +931,33 @@ public:
 	//! grabs fd's for the sockets
 	bool CreateSocksFD()
 	{
-		if( m_iReadSock != -1 )
+		if( m_iReadSock != CS_INVALID_SOCK )
 			return( true );
 
-		m_iReadSock = m_iWriteSock = SOCKET();
-		if ( m_iReadSock == -1 )
+		m_iReadSock = m_iWriteSock = CreateSocket();
+		if ( m_iReadSock == CS_INVALID_SOCK )
 			return( false );
 
 		m_address.SinFamily();
-		m_address.SinPort( m_iport );
+		m_address.SinPort( m_uPort );
 
 		return( true );
 	}
 
+	//! puts the socks back to the state they were prior to calling CreateSocksFD
+	void CloseSocksFD();
+
 	const CS_STRING & GetBindHost() const { return( m_sBindHost ); }
 	void SetBindHost( const CS_STRING & sBindHost ) { m_sBindHost = sBindHost; }
 
 	enum EDNSLType
 	{
-		DNS_VHOST,
-		DNS_DEST
+		DNS_VHOST, //!< this lookup is for the vhost bind
+		DNS_DEST //!< this lookup is for the destination address
 	};
 
 	/**
-	 * nonblocking dns lookup (when -pthread is set to compile)
+	 * dns lookup @see EDNSLType
 	 * @return 0 for success, EAGAIN to check back again (same arguments as before), ETIMEDOUT on failure
 	 */
 	int DNSLookup( EDNSLType eDNSLType );
@@ -966,13 +995,20 @@ public:
 	 */
 	virtual int GetAddrInfo( const CS_STRING & sHostname, CSSockAddr & csSockAddr );
 
+#ifdef HAVE_C_ARES
+	CSSockAddr * GetCurrentAddr() const { return( m_pCurrAddr ); }
+	void SetAresFinished( int status ) { m_pCurrAddr = NULL; m_iARESStatus = status; }
+	ares_channel GetAresChannel() { return( m_pARESChannel ); }
+#endif /* HAVE_C_ARES */
+
 private:
 	//! making private for safety
 	Csock( const Csock & cCopy ) {}
 
 	// NOTE! if you add any new members, be sure to add them to Copy()
-	u_short		m_iport, m_iRemotePort, m_iLocalPort;
-	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iMethod, m_iTcount;
+	u_short		m_uPort, m_iRemotePort, m_iLocalPort;
+	cs_sock_t	m_iReadSock, m_iWriteSock;
+	int m_itimeout, m_iConnType, m_iMethod, m_iTcount;
 	bool		m_bssl, m_bIsConnected, m_bBLOCK, m_bFullsslAccept;
 	bool		m_bsslEstablished, m_bEnableReadLine, m_bPauseRead;
 	CS_STRING	m_shostname, m_sbuffer, m_sSockName, m_sPemFile, m_sCipherType, m_sParentName;
@@ -980,7 +1016,8 @@ private:
 	ECloseType	m_eCloseType;
 
 	unsigned long long	m_iMaxMilliSeconds, m_iLastSendTime, m_iBytesRead, m_iBytesWritten, m_iStartTime;
-	unsigned int		m_iMaxBytes, m_iLastSend, m_iMaxStoredBufferLength, m_iTimeoutType;
+	unsigned int		m_iMaxBytes, m_iMaxStoredBufferLength, m_iTimeoutType;
+	size_t				m_iLastSend;
 
 	CSSockAddr 		m_address, m_bindhost;
 	bool			m_bIsIPv6, m_bSkipConnect;
@@ -990,7 +1027,6 @@ private:
 	CS_STRING			m_sSSLBuffer;
 	SSL 				*m_ssl;
 	SSL_CTX				*m_ssl_ctx;
-	SSL_METHOD			*m_ssl_method;
 	unsigned int		m_iRequireClientCertFlags;
 
 	FPCertVerifyCB		m_pCerVerifyCB;
@@ -1003,14 +1039,20 @@ private:
 	std::vector<CCron *>		m_vcCrons;
 
 	//! Create the socket
-	int SOCKET( bool bListen = false );
-	void Init( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
+	cs_sock_t CreateSocket( bool bListen = false );
+	void Init( const CS_STRING & sHostname, u_short uPort, int itimeout = 60 );
 
 
 	// Connection State Info
 	ECONState		m_eConState;
 	CS_STRING		m_sBindHost;
 	u_int			m_iCurBindCount, m_iDNSTryCount;
+#ifdef HAVE_C_ARES
+	void FreeAres();
+	ares_channel	m_pARESChannel;
+	CSSockAddr		*m_pCurrAddr;
+	int				m_iARESStatus;
+#endif /* HAVE_C_ARES */
 
 };
 
@@ -1317,6 +1359,12 @@ public:
 				pcSock->SetIPv6( true );
 #endif /* HAVE_IPV6 */
 		}
+#ifdef HAVE_IPV6
+		else
+		{
+				pcSock->SetIPv6( true );
+		}
+#endif /* HAVE_IPV6 */
 #ifdef HAVE_LIBSSL
 		pcSock->SetSSL( cListen.GetIsSSL() );
 		if( ( cListen.GetIsSSL() ) && ( !cListen.GetPemLocation().empty() ) )
@@ -1336,9 +1384,9 @@ public:
 			AddSock( pcSock, cListen.GetSockName() );
 			if( ( piRandPort ) && ( cListen.GetPort() == 0 ) )
 			{
-				int iSock = pcSock->GetSock();
+				cs_sock_t iSock = pcSock->GetSock();
 
-				if ( iSock < 0 )
+				if ( iSock == CS_INVALID_SOCK )
 				{
 					CS_DEBUG( "Failed to attain a valid file descriptor" );
 					pcSock->Close();
@@ -1370,7 +1418,6 @@ public:
 
 			if ( ( pcSock->GetType() != T::OUTBOUND ) || ( pcSock->GetConState() == T::CST_OK ) )
 				continue;
-
 			if ( pcSock->GetConState() == T::CST_DNS )
 			{
 				if ( pcSock->DNSLookup( T::DNS_VHOST ) == ETIMEDOUT )
@@ -1459,10 +1506,9 @@ public:
 
 						CSCharBuffer cBuff( iLen );
 
-						int bytes = pcSock->Read( cBuff(), iLen );
+						cs_ssize_t bytes = pcSock->Read( cBuff(), iLen );
 
-						if ( ( bytes != T::READ_TIMEDOUT ) && ( bytes != T::READ_CONNREFUSED )
-							&& ( !pcSock->IsConnected() ) )
+						if ( bytes != T::READ_TIMEDOUT && bytes != T::READ_CONNREFUSED && bytes != T::READ_ERR && !pcSock->IsConnected() )
 						{
 							pcSock->SetIsConnected( true );
 							pcSock->Connected();
@@ -1617,7 +1663,7 @@ public:
 	}
 
 	//! returns a pointer to the FIRST sock found by filedescriptor or NULL on no match
-	virtual T * FindSockByFD( int iFD )
+	virtual T * FindSockByFD( cs_sock_t iFD )
 	{
 		for( unsigned int i = 0; i < this->size(); i++ )
 			if ( ( (*this)[i]->GetRSock() == iFD ) || ( (*this)[i]->GetWSock() == iFD ) )
@@ -1707,7 +1753,7 @@ public:
 
 	//! Get the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
 	u_long GetSelectTimeout() { return( m_iSelectWait ); }
-	//! Set the Select Timeout in MICROSECODS ( 1000 == 1 millisecond )
+	//! Set the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
 	//! Setting this to 0 will cause no timeout to happen, Select() will return instantly
 	void  SetSelectTimeout( u_long iTimeout ) { m_iSelectWait = iTimeout; }
 
@@ -1820,6 +1866,12 @@ public:
 		return( iRet );
 	}
 
+protected:
+	//! this is a strict wrapper around C-api select(). Added in the event you need to do special work here
+	virtual int Select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout)
+	{
+		return( select( nfds, readfds, writefds, exceptfds, timeout ) );
+	}
 private:
 	/**
 	* fills a map of socks to a message for check
@@ -1842,38 +1894,48 @@ private:
 		TFD_ZERO( &rfds );
 		TFD_ZERO( &wfds );
 
-		// before we go any further, Process work needing to be done on the job
-		for( unsigned int i = 0; i < this->size(); i++ )
-		{
-			Csock::ECloseType eCloseType = (*this)[i]->GetCloseType();
-			if( eCloseType == T::CLT_NOW || eCloseType == T::CLT_DEREFERENCE || ( eCloseType == T::CLT_AFTERWRITE && (*this)[i]->GetWriteBuffer().empty() ) )
-				DelSock( i-- ); // close any socks that have requested it
-			else
-				(*this)[i]->Cron(); // call the Cron handler here
-		}
-
 		bool bHasWriteable = false;
 		bool bHasAvailSocks = false;
 		unsigned long long iNOW = 0;
-
 		for( unsigned int i = 0; i < this->size(); i++ )
 		{
 			T *pcSock = (*this)[i];
 
+			Csock::ECloseType eCloseType = pcSock->GetCloseType();
+
+			if( eCloseType == T::CLT_NOW || eCloseType == T::CLT_DEREFERENCE || ( eCloseType == T::CLT_AFTERWRITE && pcSock->GetWriteBuffer().empty() ) )
+			{
+				DelSock( i-- ); // close any socks that have requested it
+				continue;
+			}
+			else
+				pcSock->Cron(); // call the Cron handler here
+		
+#ifdef HAVE_C_ARES
+			ares_channel pChannel = pcSock->GetAresChannel();
+			if( pChannel )
+			{
+				ares_fds( pChannel, &rfds, &wfds );
+				// let ares drop the timeout if it has something timing out sooner then whats in tv currently
+				ares_timeout( pChannel, &tv, &tv );
+				bHasWriteable = true;
+			}
+#endif /* HAVE_C_ARES */
+
 			if ( pcSock->GetConState() != T::CST_OK )
 				continue;
 
 			bHasAvailSocks = true;
 
-			int & iRSock = pcSock->GetRSock();
-			int & iWSock = pcSock->GetWSock();
+			cs_sock_t & iRSock = pcSock->GetRSock();
+			cs_sock_t & iWSock = pcSock->GetWSock();
 			bool bIsReadPaused = pcSock->IsReadPaused();
 			if ( bIsReadPaused )
 			{
 				pcSock->ReadPaused();
 				bIsReadPaused = pcSock->IsReadPaused(); // re-read it again, incase it changed status)
 			}
-			if ( ( iRSock < 0 ) || ( iWSock < 0 ) )
+			if ( iRSock == CS_INVALID_SOCK || iWSock == CS_INVALID_SOCK )
 			{
 				SelectSock( mpeSocks, SUCCESS, pcSock );
 				continue;	// invalid sock fd
@@ -1904,7 +1966,7 @@ private:
 					{
 						pcSock->Close();
 					}
-					if( pcSock->GetWriteBuffer().size() )
+					if( !pcSock->GetWriteBuffer().empty() )
 					{ // this means we need to write again, not everything got knocked out
 						TFD_SET( iWSock, &wfds );
 						bHasWriteable = true;
@@ -1924,18 +1986,8 @@ private:
 
 			} else
 				TFD_SET( iRSock, &rfds );
-		}
-
-		// first check to see if any ssl sockets are ready for immediate read
-		// a mini select() type deal for ssl
-		for( unsigned int i = 0; i < this->size(); i++ )
-		{
-			T *pcSock = (*this)[i];
-
-			if ( pcSock->GetConState() != T::CST_OK )
-				continue;
-
-			if ( ( pcSock->GetSSL() ) && ( pcSock->GetType() != Csock::LISTENER ) )
+			
+			if( pcSock->GetSSL() && pcSock->GetType() != Csock::LISTENER )
 			{
 				if ( ( pcSock->GetPending() > 0 ) && ( !pcSock->IsReadPaused() ) )
 					SelectSock( mpeSocks, SUCCESS, pcSock );
@@ -1956,17 +2008,26 @@ private:
 			tv.tv_sec = 0;
 		}
 
-
 		if ( bHasWriteable )
-			iSel = select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
+			iSel = Select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
 		else
-			iSel = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
+			iSel = Select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
 		if ( iSel == 0 )
 		{
 			if ( mpeSocks.empty() )
 				m_errno = SELECT_TIMEOUT;
 			else
 				m_errno = SUCCESS;
+#ifdef HAVE_C_ARES
+			// run through ares channels and process timeouts
+			for( u_long uSock = 0; uSock < this->size(); ++uSock )
+			{
+				T *pcSock = this->at( uSock );
+				ares_channel pChannel = pcSock->GetAresChannel();
+				if( pChannel )
+					ares_process_fd( pChannel, ARES_SOCKET_BAD, ARES_SOCKET_BAD );
+			}
+#endif /* HAVE_C_ARES */
 
 			return;
 		}
@@ -1979,7 +2040,8 @@ private:
 				m_errno = SUCCESS;
 
 			return;
-		} else if ( iSel == -1 )
+		} 
+		else if ( iSel == -1 )
 		{
 			if ( mpeSocks.empty() )
 				m_errno = SELECT_ERROR;
@@ -1987,7 +2049,8 @@ private:
 				m_errno = SUCCESS;
 
 			return;
-		} else
+		} 
+		else
 		{
 			m_errno = SUCCESS;
 		}
@@ -1997,14 +2060,20 @@ private:
 		{
 			T *pcSock = (*this)[i];
 
+#ifdef HAVE_C_ARES
+			ares_channel pChannel = pcSock->GetAresChannel();
+			if( pChannel )
+				ares_process( pChannel, &rfds, &wfds );
+#endif /* HAVE_C_ARES */
+
 			if ( pcSock->GetConState() != T::CST_OK )
 				continue;
 
-			int & iRSock = pcSock->GetRSock();
-			int & iWSock = pcSock->GetWSock();
+			cs_sock_t & iRSock = pcSock->GetRSock();
+			cs_sock_t & iWSock = pcSock->GetWSock();
 			EMessages iErrno = SUCCESS;
 
-			if ( ( iRSock < 0 ) || ( iWSock < 0 ) )
+			if ( iRSock == CS_INVALID_SOCK || iWSock == CS_INVALID_SOCK )
 			{
 				// trigger a success so it goes through the normal motions
 				// and an error is produced
@@ -2043,9 +2112,9 @@ private:
 				{
 					CS_STRING sHost;
 					u_short port;
-					int inSock = pcSock->Accept( sHost, port );
+					cs_sock_t inSock = pcSock->Accept( sHost, port );
 
-					if ( inSock != -1 )
+					if ( inSock != CS_INVALID_SOCK )
 					{
 						if ( T::TMO_ACCEPT & pcSock->GetTimeoutType() )
 							pcSock->ResetTimer();	// let them now it got dinged
