@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,7 +26,7 @@ public:
 	static unsigned short DCCRequest(const CString& sNick, unsigned long uLongIP, unsigned short uPort, const CString& sFileName, bool bIsChat, CUser* pUser, const CString& sRemoteIP);
 
 	void ReadLine(const CString& sData);
-	virtual void ReadData(const char* data, int len);
+	virtual void ReadData(const char* data, size_t len);
 	virtual void ReadPaused();
 	virtual void Timeout();
 	virtual void ConnectionRefused();
@@ -57,19 +57,19 @@ public:
 	// !Getters
 private:
 protected:
-	CString					m_sRemoteNick;
-	CString					m_sRemoteIP;
-	CString					m_sConnectIP;
-	CString					m_sLocalIP;
-	CString					m_sFileName;
-	CUser*					m_pUser;
-	CDCCBounce*				m_pPeer;
-	unsigned short				m_uRemotePort;
-	bool					m_bIsChat;
-	bool					m_bIsRemote;
+	CString                      m_sRemoteNick;
+	CString                      m_sRemoteIP;
+	CString                      m_sConnectIP;
+	CString                      m_sLocalIP;
+	CString                      m_sFileName;
+	CUser*                       m_pUser;
+	CDCCBounce*                  m_pPeer;
+	unsigned short               m_uRemotePort;
+	bool                         m_bIsChat;
+	bool                         m_bIsRemote;
 
-	static const unsigned int		m_uiMaxDCCBuffer;
-	static const unsigned int		m_uiMinDCCBuffer;
+	static const unsigned int    m_uiMaxDCCBuffer;
+	static const unsigned int    m_uiMinDCCBuffer;
 };
 
 #endif // !_DCCBOUNCE_H
