@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -53,7 +53,7 @@ public:
 	bool IsSock(bool bUseLstat = false) const;
 
 	// for gettin file types, using fstat instead
-	static bool FType(const CString sFileName, EFileTypes eType, bool bUseLstat = false);
+	static bool FType(const CString& sFileName, EFileTypes eType, bool bUseLstat = false);
 
 	enum EFileAttr {
 		FA_Name,
@@ -122,12 +122,12 @@ private:
 	// flock() wrapper
 	bool Lock(int iOperation);
 
-	CString	m_sBuffer;
-	int		m_iFD;
+	CString m_sBuffer;
+	int     m_iFD;
 
 protected:
-	CString	m_sLongName;	//!< Absolute filename (m_sPath + "/" + m_sShortName)
-	CString	m_sShortName;	//!< Filename alone, without path
+	CString m_sLongName;  //!< Absolute filename (m_sPath + "/" + m_sShortName)
+	CString m_sShortName; //!< Filename alone, without path
 };
 
 class CDir : public vector<CFile*> {
@@ -160,46 +160,6 @@ public:
 		return FillByWildcard(sDir, "*");
 	}
 
-/*	static bool Create(const CString& sDir, mode_t mode = 0755) {
-		VCCString vSubDirs = sDir.split("[/\\\\]+");
-		CCString sCurDir;
-
-		for (unsigned int a = 0; a < vSubDirs.size(); a++) {
-			sCurDir += vSubDirs[a] + "/";
-			if ((!CDir::Exists(sCurDir)) && (mkdir(sCurDir.c_str(), mode) != 0)) {
-				return false;
-			}
-		}
-
-		return true;
-	}
-
-	int FillByRegex(const CCString& sDir, const CCString& sRegex, const CCString& sFlags = "") {
-		CleanUp();
-		DIR* dir = opendir((sDir.empty()) ? "." : sDir.c_str());
-
-		if (!dir) {
-			return 0;
-		}
-
-		struct dirent * de;
-
-		while ((de = readdir(dir)) != 0) {
-			if ((strcmp(de->d_name, ".") == 0) || (strcmp(de->d_name, "..") == 0)) {
-				continue;
-			}
-			if ((!sRegex.empty()) && (!CCString::search(de->d_name, sRegex, sFlags))) {
-				continue;
-			}
-
-			CFile *file = new CFile(sDir, de->d_name, this);
-			push_back(file);
-		}
-
-		closedir(dir);
-		return size();
-	}*/
-
 	int FillByWildcard(const CString& sDir, const CString& sWildcard) {
 		CleanUp();
 		DIR* dir = opendir((sDir.empty()) ? "." : sDir.c_str());
@@ -218,7 +178,7 @@ public:
 				continue;
 			}
 
-			CFile *file = new CFile(sDir + "/" + de->d_name/*, this*/);	// @todo need to pass pointer to 'this' if we want to do Sort()
+			CFile *file = new CFile(sDir + "/" + de->d_name/*, this*/); // @todo need to pass pointer to 'this' if we want to do Sort()
 			push_back(file);
 		}
 
@@ -282,8 +242,8 @@ public:
 
 private:
 protected:
-	CFile::EFileAttr	m_eSortAttr;
-	bool				m_bDesc;
+	CFile::EFileAttr m_eSortAttr;
+	bool             m_bDesc;
 };
 
 //! @author imaginos@imaginos.net
@@ -316,6 +276,6 @@ public:
 	void close2(int iPid, int iReadFD, int iWriteFD);
 
 private:
-	int			m_iPid;
+	int  m_iPid;
 };
 #endif // !_FILEUTILS_H
